clear all
close all

s = pwd;

addpath([s,'/DIFFOP'])
addpath([s,'/ASSEMBLE'])
addpath([s,'/GEOM2D']);

disp(' ')
disp('A two dimensional Poisson solver')
disp('based on the EJIIM')
disp(' ')
disp('parameters of the inclusion see in')
disp('     ASSEMBLE/problem_setup.m')
disp('the right hand side function and the')
disp('boundary data are in')
disp('     ASSEMBLE/set_rhs.m')
disp(' ')
poisson_solve



