% part of test_interface

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SETUP

% embedding box: [ax,bx]*[ay,by]
% because of the geometry preprocessor, it has to be a square
ax = -0.5; bx = 0.5; 
ay = ax;   by = bx; 

% set Dirichlet boundary condition on the interface portion
% with x<x_level
x_level = 0.3;

% grid parameters
n = 41;
% because of the geometry pre-processor assume equidistant mesh
nx = n; ny = n;

disp(['number of grid points per direction: ',num2str(n)]);

GEOM = 'circle'; % 'circle' or 'flower'
        
% parameters which can be set are
% (xc, yc) - the origin of the circle
% R: the radius of the circle, which for the 'flower' is then later
% perturbed

xc = 0.001; yc = -0.001; R = 0.45;

