%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the right hand side data
% write the rhs as function of X and Y
F = GRID.X.^2 + GRID.Y;

% in the extension F is zero, thus
F = F.*M; F = reshape(F,GRID.nx*GRID.ny,1);

% data across the boundary
% coordinates of the intersection points
% are IX.coord(:,1) (x-coordinate) and IX.coord(:,2) (y-coordinate)

% the jump in the right hand side at the boundary
% (= the rhs with the minus sign)
jf = -IX.coord(:,1).^2 + IX.coord(:,2);

% Dirichlet b.c.
% coordinates of the intersection points are given by the
% variables IX_D.coord(:,1) (x-coordinate) and IX_D.coord(:,2) (y-coordinate)
uD = IX_D.coord(:,1).^2;

% Neumann b.c.
% coordinates of the intersection points are given by the
% variables IX_N.coord(:,1) (x-coordinate) and IX_N.coord(:,2) (y-coordinate)
uN = IX_N.n(:,1) + IX_N.n(:,2);
